﻿using System;
using System.Linq;
using System.Security.Claims;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.Controllers
{
    [Authorize]
    [Route("api/[controller]")]
    [ApiController]
    public class CourseController : ControllerBase
    {
        private readonly ICourseService _courseService;
        private readonly IUserService _userService;
        private readonly IBedeliaService _bedeliaService;
        private readonly IStudentCourseResultService _studentCourseResultService;

        public CourseController(
            ICourseService courseService,
            IUserService userService,
            IBedeliaService bedeliaService,
            IStudentCourseResultService studentCourseResultService)
        {
            _courseService = courseService;
            _userService = userService;
            _bedeliaService = bedeliaService;
            _studentCourseResultService = studentCourseResultService;
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpGet("MyCourses")]
        public async Task<IActionResult> MyCourses()
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);
            var userType = HttpContext.User.Claims.FirstOrDefault(x => x.Type == ClaimTypes.Role).Value;

            var result = await _courseService.GetCoursePreviews(userId, userType);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpPost("Create")]
        public async Task<IActionResult> Create(CourseCreateDto newCourse)
        {
            var tenantId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "TenantId").Value);
            newCourse.TenantId = tenantId;

            var bedeliaResult = await _bedeliaService.IsValidCourse(newCourse.Name);
            if (bedeliaResult.HasError)
            {
                return BadRequest(bedeliaResult.Message);
            }

            var result = await _courseService.Create(newCourse);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student)]
        [HttpPost("Matriculate")]
        public async Task<IActionResult> Matriculate(CourseMatriculateDto courseMatriculateDto)
        {
            try
            {
                var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);
                var user = await _userService.GetById(userId);

                var response = await _bedeliaService.IsValidUser(user.Data.Student.IdentityCard);
                if (response.HasError)
                {
                    return BadRequest(response.Message);
                }

                var result = await _courseService.Matriculate(userId, courseMatriculateDto);
                if (result.HasError)
                {
                    return BadRequest(result.Message);
                }

                if (result.Data == false)
                {
                    return NotFound(result.Message);
                }
            }
            catch (Exception ex)
            {
                return BadRequest(ex.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.Student)]
        [HttpPost("DropOutFromCourse/{courseId}")]
        public async Task<IActionResult> DropOutFromCourse(int courseId)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);

            var result = await _courseService.DropOutFromCourse(userId, courseId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            if (result.Data == false)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.FacultyAdmin + ", " + UserTypes.Professor)]
        [HttpPost("AddProfessorToCourse")]
        public async Task<IActionResult> AddProfessorToCourse(ProfessorCourseDto professorCourseDto)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);
            var userType = HttpContext.User.Claims.FirstOrDefault(x => x.Type == ClaimTypes.Role).Value;

            var result = await _courseService.AddProfessorToCourse(professorCourseDto, userId, userType);
            
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            if (result.Data == false)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpPost("RemoveProfessorToCourse")]
        public async Task<IActionResult> RemoveProfessorFromCourse(ProfessorCourseDto professorCourseDto)
        {
            var result = await _courseService.RemoveProfessorToCourse(professorCourseDto);
            
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            if (result.Data == false)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor + ", " + UserTypes.FacultyAdmin)]
        [HttpGet("GetAll")]
        public async Task<IActionResult> GetAll()
        {
            var tenantId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "TenantId").Value);

            var result = await _courseService.GetAll(tenantId);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.UdelarAdmin)]
        [HttpGet("GetAll/{tenantId}")]
        public async Task<IActionResult> GetAll(int tenantId)
        {
            var result = await _courseService.GetAll(tenantId);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpGet("GetAllTemplates")]
        public async Task<IActionResult> GetAllTemplates()
        {
            var tenantId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "TenantId").Value);

            var result = await _courseService.GetAll(tenantId, true);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Professor + ", " + UserTypes.FacultyAdmin)]
        [HttpPut("Modify/{courseId}")]
        public async Task<IActionResult> Modify(int courseId, CourseModifyDto courseDto)
        {
            var tenantId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "TenantId").Value);
            courseDto.TenantId = tenantId;

            var result = await _courseService.Modify(courseId, courseDto);
            
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            if (result.Data == false)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpDelete("Delete/{courseId}")]
        public async Task<IActionResult> Delete(int courseId)
        {
            var result = await _courseService.Delete(courseId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            if (result.Data == null)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor + ", " + UserTypes.FacultyAdmin + ", " + UserTypes.UdelarAdmin)]
        [HttpGet("GetProfessors/{courseId}")]
        public async Task<IActionResult> GetProfessors(int courseId)
        {
            var result = await _courseService.GetProfessors(courseId);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor + ", " + UserTypes.FacultyAdmin + ", " + UserTypes.UdelarAdmin)]
        [HttpGet("GetStudents/{courseId}")]
        public async Task<IActionResult> GetStudents(int courseId)
        {
            var result = await _courseService.GetStudents(courseId);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor + ", " + UserTypes.FacultyAdmin)]
        [HttpGet("GetById/{courseId}")]
        public async Task<IActionResult> GetById(int courseId)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);
            var myUserType = HttpContext.User.Claims.FirstOrDefault(x => x.Type == ClaimTypes.Role).Value;

            int? studentId = null;
            if (myUserType == UserTypes.Student)
            {
                var userResult = await _userService.GetById(userId);
                if (userResult.HasError)
                {
                    return BadRequest(userResult.Message);
                }

                studentId = userResult.Data.Student.Id;
            }

            var result = await _courseService.GetById(courseId, studentId);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.FacultyAdmin)]
        [HttpPost("CloseRecord/{courseId}")]
        public async Task<IActionResult> CloseRecord(int courseId)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);

            var courseResult = await _courseService.GetById(courseId);
            if (courseResult.HasError)
            {
                return BadRequest(courseResult.Message);
            }

            var studentCourseResult = await _studentCourseResultService.GetLastStudentCourseResults(courseId, userId, false);
            if (studentCourseResult.HasError)
            {
                return BadRequest(studentCourseResult.Message);
            }

            var userResult = _userService.GetUserGrades(studentCourseResult.Data);
            if (userResult.HasError)
            {
                return BadRequest(userResult.Message);
            }

            var courseName = courseResult.Data.Name;
            var userGrades = userResult.Data;

            var result = await _bedeliaService.CloseRecord(courseName, userGrades);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok();
        }
    }
}
